import {AfterContentInit, ContentChild, Directive, EventEmitter, HostListener, Input, Output} from '@angular/core';
import {IDataSource} from '@openharbor/data';
import {finalize} from "rxjs";
import {
  ConfirmOptions, FormType,
  ICommandDirectiveService,
  IConfirmOptions
} from "../abstractions/command-directive-service.abstraction";
import {CommandFormDirective} from "./command-form.directive";

@Directive({
  selector: '[duiCommand]',
  standalone: true
})
export class CommandDirective<TModel extends {}, TConfirmOptions extends IConfirmOptions, TForm extends FormType<TForm> = any> implements AfterContentInit {
  @ContentChild(CommandFormDirective) commandFormTemplate?: CommandFormDirective<TForm>;

  @Input() confirm: boolean = false;
  @Input() confirmOptions?: TConfirmOptions;
  @Input() refresh: boolean = true;
  @Input() params?: any;

  @Input({ required: true }) dataSource!: IDataSource<TModel>;
  @Input({ required: true }) command!: string;
  @Input() model: object = {};
  @Input() service?: ICommandDirectiveService;

  @Output() success: EventEmitter<any> = new EventEmitter<any>();
  @Output() failure: EventEmitter<any> = new EventEmitter<any>();
  @Output() loading: EventEmitter<boolean> = new EventEmitter<boolean>();

  ngAfterContentInit(): void {
    console.log('init');
  }

  @HostListener('click')
  handleClick() {
    if (this.confirm) {
      if (!this.service) {
        const error = new Error(`No service provided to directive for CommandDirectiveService!`);
        error.name = 'NullInjectorError';
        throw error;
      }

      const options = {
        ...this.confirmOptions,
        ...{
          success: this.success.asObservable(),
          failure: this.failure.asObservable(),
          loading: this.loading.asObservable(),
        },
        ...{
          formTemplate: this.commandFormTemplate?.template,
          form: this.commandFormTemplate?.form
        }
      } as ConfirmOptions<TConfirmOptions>;

      this.service.confirm(options)
        .subscribe(result => {
          const model = { ...this.model, ...options.form?.getRawValue() };
          console.log(options.form?.getRawValue());
          if (result)
            this.executeCommand(model);
        });
      return;
    }

    this.executeCommand();
  }

  private executeCommand(model?: any) {
    this.dataSource.resolveCommandModelByName<TModel>({
      command: this.command,
      model: model,
      params: this.params
    })
      .subscribe({ next: commandModel => {
        this.loading.emit(true);
        this.dataSource.executeCommandByName(this.command, commandModel)
          .pipe(
            finalize(() => this.loading.emit(false))
          )
          .subscribe({
            next: commandResult =>
            {
              if (this.refresh)
                this.dataSource.refresh();

              this.success.emit(commandResult);
            },
            error: commandError => {
              this.failure.emit(commandError);
            }
          });
      },
      error: resolveCommandError => {
        this.failure.emit(resolveCommandError);
      }
    });
  }
}
