import {Component, EventEmitter, inject, OnDestroy, OnInit, TemplateRef} from '@angular/core';
import {
  MAT_DIALOG_DATA,
  MatDialogActions,
  MatDialogClose,
  MatDialogContent,
  MatDialogRef,
  MatDialogTitle
} from "@angular/material/dialog";
import {MatButton} from "@angular/material/button";
import {AsyncPipe, NgTemplateOutlet} from "@angular/common";
import {MatProgressSpinner} from "@angular/material/progress-spinner";
import {Observable, Subscription} from "rxjs";
import {FormGroup} from "@angular/forms";
import {FormType} from "@openharbor/ngx-data-ui-core";

export interface IConfirmDialogDefaultOptions<TControl extends FormType<TControl> = any> {
  title: string;
  message: string;
  confirmText?: string;
  cancelText?: string;
  success: Observable<any>;
  failure: Observable<any>;
  loading: Observable<boolean>;
  formTemplate?: TemplateRef<any>;
  form?: FormGroup<TControl>;
}

@Component({
  standalone: true,
  imports: [
    MatDialogContent,
    MatDialogActions,
    MatButton,
    MatDialogClose,
    MatDialogTitle,
    AsyncPipe,
    MatProgressSpinner,
    NgTemplateOutlet
  ],
  templateUrl: './confirm-dialog-default.component.html',
  styleUrl: './confirm-dialog-default.component.css'
})
export class ConfirmDialogDefaultComponent<TControl extends FormType<TControl> = any> implements OnInit, OnDestroy {
  readonly data = inject<IConfirmDialogDefaultOptions<TControl>>(MAT_DIALOG_DATA);
  readonly ref = inject(MatDialogRef<ConfirmDialogDefaultComponent>);

  readonly title: string = this.data.title;
  readonly message: string = this.data.message;
  readonly confirmText?: string = this.data.confirmText;
  readonly cancelText?: string = this.data.cancelText;
  readonly $loading: Observable<boolean> = this.data.loading;
  readonly success: Observable<any> = this.data.success;
  readonly formTemplate?: TemplateRef<any> = this.data.formTemplate;
  readonly form?: FormGroup<TControl>;

  isLoading = false;
  // todo: error messaging?

  private subscriptions: Subscription[] = [];
  private _onConfirm = new EventEmitter<void>;
  private _disableCloseOriginalValue?: boolean;

  get onConfirm() {
    return this._onConfirm.asObservable();
  }

  ngOnInit(): void {
    this._disableCloseOriginalValue = this.ref.disableClose;
    this.subscriptions.push(this.$loading.subscribe(isLoading => {
      this.isLoading = isLoading;
      this.ref.disableClose = isLoading ? true : this._disableCloseOriginalValue;
    }));

    this.subscriptions.push(this.success.subscribe(_ => {
      this.ref.close();
    }));
  }

  ngOnDestroy() {
    for (let subscription of this.subscriptions)
      subscription.unsubscribe();
  }

  onConfirmed() {
    if (!this.formTemplate || !this.form) {
      this._onConfirm.emit();
      return;
    }

    this.form.clearValidators();

    if (!this.form.valid) {
      this.form.markAllAsTouched();
      return;
    }

    this._onConfirm.emit();
  }
}
