import {Component, Input, OnDestroy, OnInit} from '@angular/core';
import {IDataSource} from "@openharbor/data";
import {
  MatCell,
  MatCellDef,
  MatColumnDef,
  MatHeaderCell,
  MatHeaderCellDef,
  MatHeaderRow,
  MatHeaderRowDef,
  MatRow,
  MatRowDef,
  MatTable
} from "@angular/material/table";
import {MatPaginator, PageEvent} from "@angular/material/paginator";
import {map, Observable, Subscription, tap} from "rxjs";
import {MatProgressBar} from "@angular/material/progress-bar";
import {AsyncPipe, JsonPipe, TitleCasePipe} from "@angular/common";
import {MatSort, MatSortHeader, Sort} from "@angular/material/sort";

export interface IDataGridOptions<TModel extends object> {
  columns: IColumnDefinition<TModel, Extract<keyof TModel, string>>[];
  actions?: [];
  pagination: {
    enable: boolean;
    pageSize: number;
    hidePageSize: boolean;
    pageSizeOptions: number[];
  }
}

export interface IColumnDefinition<TModel extends object, TColumn extends Extract<keyof TModel, string>> {
  column: TColumn;
  sorting?: {
    enable: boolean;
    description?: (element: TColumn) => Observable<string>;
    path?: string;
  },
  title?: (element: TColumn) => Observable<string>;
  value?: (element: TModel[TColumn]) => Observable<string|number|boolean>;
}

export interface IRowAction<TModel extends object> {

}

@Component({
  selector: 'mdx-data-grid',
  standalone: true,
  imports: [
    MatTable,
    MatColumnDef,
    MatHeaderCell,
    MatCell,
    MatPaginator,
    MatHeaderRow,
    MatHeaderRowDef,
    MatRow,
    MatRowDef,
    MatProgressBar,
    MatHeaderCellDef,
    MatCellDef,
    TitleCasePipe,
    JsonPipe,
    AsyncPipe,
    MatSort,
    MatSortHeader
  ],
  templateUrl: './data-grid.component.html',
  styleUrl: './data-grid.component.css'
})
export class DataGridComponent<TModel extends object> implements OnInit, OnDestroy {
  @Input({ required: true }) dataSource!: IDataSource<TModel>;
  @Input() options?: Partial<IDataGridOptions<TModel>>;

  private subscriptions: Subscription[] = [];

  $data?: Observable<TModel[]>;
  totalRecords: number = 0;
  isLoading: boolean = false;

  _options: IDataGridOptions<TModel> = {
    columns: [],
    pagination: {
      enable: true,
      hidePageSize: false,
      pageSizeOptions: [10, 25, 50, 100],
      pageSize: 10
    }
  };

  getColumns() {
    return this._options.columns.map(options => {
      return options.column;
    });
  }

  ngOnInit() {
    this._options = {
      ...this._options,
      ...this.options
    };

    this.setPageSize(this._options.pagination.pageSize);

    this.subscriptions.push(this.dataSource.loading$
      .subscribe(loading => this.isLoading = loading)
    );

    this.$data = this.dataSource.data$
      .pipe(
        tap((result => {
          this.totalRecords = result?.totalRecords ?? 0;
        })),
        map((result) => {
          return result?.data ?? [];
        })
      );
  }

  ngOnDestroy() {
    for (let subscription of this.subscriptions)
      subscription.unsubscribe();
  }

  onPaginationEvent(ev: PageEvent) {
    this.setPageSize(ev.pageSize);
    this.setPage(ev.pageIndex);
    this.refresh();
  }

  setPage(index: number) {
    this.dataSource.page = index + 1;
  }

  setPageSize(size: number) {
    this._options.pagination.pageSize = size;
    this.dataSource.pageSize = size;
  }

  refresh() {
    this.dataSource.refresh();
  }

  onSortChanged(state: Sort) {
    if (state.direction === '') {
      this.dataSource.sorts = this.dataSource.sorts.filter(sort => sort.path !== state.active);
    } else {
      let field = this.dataSource.sorts.find(field => field.path === state.active);

      if (field === undefined) {
        field = {
          path: state.active
        };
        this.dataSource.sorts.push(field);
      }


      field.ascending = state.direction === 'asc';
    }

    this.refresh();
  }
}
